@@ -15,30 +15,23 @@ $stepfunc[6] = 'init_database_dynamic';
 $stepfunc[7] = 'congrats';
 
 if (isset ($_REQUEST['step']))
-	$step = $_REQUEST['step'];
+	$step = intval ($_REQUEST['step']);
 else
 	$step = 1;
 
-if ($step > count ($stepfunc))
+if (! array_key_exists ($step, $stepfunc))
 {
-	$root = (empty ($_SERVER['HTTPS']) || $_SERVER['HTTPS'] == 'off') ? 'http://' : 'https://';
-	$root .= isset ($_SERVER['HTTP_HOST']) ? $_SERVER['HTTP_HOST'] : ($_SERVER['SERVER_NAME'].($_SERVER['SERVER_PORT']=='80'?'':$_SERVER['SERVER_PORT']));
-	// "Since PHP 4.3.0, you will often get a slash or a dot back from
-	// dirname() in situations where the older functionality would have given
-	// you the empty string."
-	// "On Windows, both slash (/) and backslash (\) are used as directory
-	// separator character."
-	$root .= strtr (dirname ($_SERVER['PHP_SELF']), '\\', '/');
-	if (substr ($root, -1) != '/')
-		$root .= '/';
-	header ("Location: ${root}");
+	// Leave the installer module.
+	header ('Location: ' . $_SERVER['PHP_SELF']);
 	exit;
 }
 $title = "RackTables installation: step ${step} of " . count ($stepfunc);
 header ('Content-Type: text/html; charset=UTF-8');
-?><!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
+	// Heredoc, not nowdoc!
+	echo <<<"ENDOFTEXT"
+<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
 <html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
-<head><title><?php echo $title; ?></title>
+<head><title>${title}</title>
 <style type="text/css">
 .tdleft {
 	text-align: left;
@@ -59,7 +52,7 @@ header ('Content-Type: text/html; charset=UTF-8');
 </head>
 <body>
 <center>
-<?php
+ENDOFTEXT;
 echo "<h1>${title}</h1><p>";
 
 echo "</p><form method=post>\n";
@@ -75,14 +68,10 @@ else
 	echo "<br><input type=submit value='retry'>";
 }
 echo "<input type=hidden name=step value='${next_step}'>\n";
-
-?>
-</form>
-</center>
-</body>
-</html>
-
-<?php
+	echo '</form>';
+	echo '</center>';
+	echo '</body>';
+	echo '</html>';
 }
 
 // Check if the software is already installed.
@@ -96,16 +85,16 @@ function not_already_installed()
 	}
 	else
 	{
-		echo 'There seem to be no existing installation here, I am going to setup one now.<br>';
+		echo 'There seems to be no existing installation here, let\'s set one up now.<br>';
 		return TRUE;
 	}
 }
 
-// Check that we can write to configuration file.
-// If so, ask for DB connection paramaters and test
-// the connection. Neither save the parameters nor allow
-// going further until we succeed with the given
-// credentials.
+// Test that the web-server can write to the configuration file.
+// If so, prompt for the DB connection parameters and test
+// the connection. Do not save the parameters into the configuration
+// file until the database connection succeeds. Do not proceed to the
+// next steps until a working configuration file is in place.
 function init_config ()
 {
 	function print_form
@@ -251,24 +240,27 @@ function init_config ()
 	fwrite ($conf, "\$pdo_dsn = '${pdo_dsn}';\n");
 	fwrite ($conf, "\$db_username = '" . $_REQUEST['mysql_username'] . "';\n");
 	fwrite ($conf, "\$db_password = '" . $_REQUEST['mysql_password'] . "';\n\n");
-	fwrite ($conf, <<<ENDOFTEXT
+	fwrite ($conf, <<<'ENDOFTEXT'
+# Set this if you need to override the default plugins directory.
+#$racktables_plugins_dir = '/path/to/plugins';
+
 # Setting MySQL client buffer size may be required to make downloading work for
 # larger files, but it does not work with mysqlnd.
-# \$pdo_bufsize = 50 * 1024 * 1024;
+# $pdo_bufsize = 50 * 1024 * 1024;
 # Setting PDO SSL key, cert, and CA will allow a SSL/TLS connection to the MySQL
 # DB. Make sure the files are readable by the web server
-# \$pdo_ssl_key = '/path/to/ssl/key'
-# \$pdo_ssl_cert = '/path/to/ssl/cert'
-# \$pdo_ssl_ca = '/path/to/ssl/ca'
+# $pdo_ssl_key = '/path/to/ssl/key'
+# $pdo_ssl_cert = '/path/to/ssl/cert'
+# $pdo_ssl_ca = '/path/to/ssl/ca'
 
-\$user_auth_src = 'database';
-\$require_local_account = TRUE;
+$user_auth_src = 'database';
+$require_local_account = TRUE;
 # Default setting is to authenticate users locally, but it is possible to
 # employ existing LDAP or Apache user accounts. Check RackTables wiki for
 # more information, in particular, this page for LDAP configuration details:
-# http://wiki.racktables.org/index.php?title=LDAP
+# https://wiki.racktables.org/index.php?title=LDAP
 
-#\$LDAP_options = array
+#$LDAP_options = array
 #(
 #	'server' => 'localhost',
 #	'domain' => 'example.com',
@@ -283,9 +275,9 @@ function init_config ()
 #);
 
 # For SAML configuration details:
-# http://wiki.racktables.org/index.php?title=SAML
+# https://wiki.racktables.org/index.php?title=SAML
 
-#\$SAML_options = array
+#$SAML_options = array
 #(
 #	'simplesamlphp_basedir' => '../simplesaml',
 #	'sp_profile' => 'default-sp',
@@ -299,12 +291,10 @@ function init_config ()
 # be appended to assorted error messages visible in user's browser (including
 # "not authenticated" message). Beware of placing any sensitive information
 # here, it will be readable by unauthorized visitors.
-#\$helpdesk_banner = '<B>This RackTables instance is supported by Example Inc. IT helpdesk, dial ext. 1234 to report a problem.</B>';
-
+#$helpdesk_banner = '<B>This RackTables instance is supported by Example Inc. IT helpdesk, dial ext. 1234 to report a problem.</B>';
 
 ENDOFTEXT
 );
-	fwrite ($conf, "?>\n");
 	fclose ($conf);
 	echo "The configuration file has been written successfully.<br>";
 	return TRUE;
@@ -327,7 +317,7 @@ function check_config_access()
 	global $path_to_secret_php;
 	if (! is_writable ($path_to_secret_php) && is_readable ($path_to_secret_php))
 	{
-		echo 'The configuration file ownership/permissions seem to be OK.<br>';
+		echo 'The configuration file ownership and permissions seem to be OK.<br>';
 		return TRUE;
 	}
 	$uname = get_process_owner();
@@ -362,6 +352,8 @@ function init_database_static ()
 {
 	connect_to_db_or_die();
 	global $dbxlink;
+	// platform_is_ok() didn't check for InnoDB support during its previous invocation, which
+	// was right because at that time secret.php did not exist yet and $dbxlink was not available.
 	if (!isInnoDBSupported())
 	{
 		echo 'InnoDB test failed! Please configure MySQL server properly and retry.';
@@ -473,9 +465,9 @@ function congrats ()
 {
 	echo 'Congratulations! RackTables installation is complete. After pressing Proceed you will ';
 	echo 'enter the system. Authenticate with <strong>admin</strong> username.<br>RackTables project has a ';
-	echo "<a href='http://wiki.racktables.org/index.php?title=RackTablesAdminGuide'>";
+	echo "<a href='https://wiki.racktables.org/index.php?title=RackTablesAdminGuide'>";
 	echo "wiki</a> and a ";
-	echo "<a href='http://www.freelists.org/list/racktables-users'>mailing list</a> for users. Have fun.<br>";
+	echo "<a href='https://www.freelists.org/list/racktables-users'>mailing list</a> for users. Have fun.<br>";
 	return TRUE;
 }
 
@@ -491,10 +483,12 @@ function get_pseudo_file ($name)
 		$query[] = "SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0";
 
 		$query[] = "CREATE TABLE `Atom` (
-  `molecule_id` int(10) unsigned default NULL,
-  `rack_id` int(10) unsigned default NULL,
-  `unit_no` int(10) unsigned default NULL,
-  `atom` enum('front','interior','rear') default NULL,
+  `molecule_id` int(10) unsigned NOT NULL,
+  `rack_id` int(10) unsigned NOT NULL,
+  `unit_no` int(10) unsigned NOT NULL,
+  `atom` enum('front','interior','rear') NOT NULL,
+  PRIMARY KEY (`molecule_id`,`rack_id`,`unit_no`,`atom`),
+  KEY `Atom-FK-rack_id` (`rack_id`),
   CONSTRAINT `Atom-FK-molecule_id` FOREIGN KEY (`molecule_id`) REFERENCES `Molecule` (`id`) ON DELETE CASCADE,
   CONSTRAINT `Atom-FK-rack_id` FOREIGN KEY (`rack_id`) REFERENCES `Object` (`id`) ON DELETE CASCADE
 ) ENGINE=InnoDB";
@@ -564,26 +558,6 @@ function get_pseudo_file ($name)
   CONSTRAINT `CachedPVM-FK-object_id` FOREIGN KEY (`object_id`) REFERENCES `Object` (`id`) ON DELETE CASCADE
 ) ENGINE=InnoDB";
 
-		$query[] = "CREATE TABLE `CactiGraph` (
-  `object_id` int(10) unsigned NOT NULL,
-  `server_id` int(10) unsigned NOT NULL,
-  `graph_id` int(10) unsigned NOT NULL,
-  `caption`  char(255) DEFAULT NULL,
-  PRIMARY KEY (`object_id`,`server_id`,`graph_id`),
-  KEY `graph_id` (`graph_id`),
-  KEY `server_id` (`server_id`),
-  CONSTRAINT `CactiGraph-FK-server_id` FOREIGN KEY (`server_id`) REFERENCES `CactiServer` (`id`),
-  CONSTRAINT `CactiGraph-FK-object_id` FOREIGN KEY (`object_id`) REFERENCES `Object` (`id`) ON DELETE CASCADE
-) ENGINE=InnoDB";
-
-		$query[] = "CREATE TABLE `CactiServer` (
-  `id` int(10) unsigned NOT NULL auto_increment,
-  `base_url` char(255) DEFAULT NULL,
-  `username` char(64) DEFAULT NULL,
-  `password` char(64) DEFAULT NULL,
-  PRIMARY KEY (`id`)
-) ENGINE=InnoDB";
-
 		$query[] = "CREATE TABLE `Chapter` (
   `id` int(10) unsigned NOT NULL auto_increment,
   `sticky` enum('yes','no') default 'no',
@@ -662,7 +636,7 @@ function get_pseudo_file ($name)
   `object_id` int(10) unsigned NOT NULL default '0',
   `ip` int(10) unsigned NOT NULL default '0',
   `name` char(255) NOT NULL default '',
-  `type` enum('regular','shared','virtual','router','point2point') NOT NULL DEFAULT 'regular',
+  `type` enum('regular','shared','virtual','router','point2point','sharedrouter') NOT NULL DEFAULT 'regular',
   PRIMARY KEY  (`object_id`,`ip`),
   KEY `ip` (`ip`),
   CONSTRAINT `IPv4Allocation-FK-object_id` FOREIGN KEY (`object_id`) REFERENCES `Object` (`id`) ON DELETE CASCADE
@@ -774,7 +748,7 @@ function get_pseudo_file ($name)
   `object_id` int(10) unsigned NOT NULL default '0',
   `ip` binary(16) NOT NULL,
   `name` char(255) NOT NULL default '',
-  `type` enum('regular','shared','virtual','router','point2point') NOT NULL DEFAULT 'regular',
+  `type` enum('regular','shared','virtual','router','point2point','sharedrouter') NOT NULL DEFAULT 'regular',
   PRIMARY KEY  (`object_id`,`ip`),
   KEY `ip` (`ip`),
   CONSTRAINT `IPv6Allocation-FK-object_id` FOREIGN KEY (`object_id`) REFERENCES `Object` (`id`) ON DELETE CASCADE
@@ -827,30 +801,14 @@ function get_pseudo_file ($name)
   `new_molecule_id` int(10) unsigned default NULL,
   `comment` text,
   PRIMARY KEY  (`id`),
+  UNIQUE KEY `old_molecule_id` (`old_molecule_id`),
+  UNIQUE KEY `new_molecule_id` (`new_molecule_id`),
   KEY `object_id` (`object_id`),
   CONSTRAINT `MountOperation-FK-object_id` FOREIGN KEY (`object_id`) REFERENCES `Object` (`id`) ON DELETE CASCADE,
   CONSTRAINT `MountOperation-FK-old_molecule_id` FOREIGN KEY (`old_molecule_id`) REFERENCES `Molecule` (`id`) ON DELETE CASCADE,
   CONSTRAINT `MountOperation-FK-new_molecule_id` FOREIGN KEY (`new_molecule_id`) REFERENCES `Molecule` (`id`) ON DELETE CASCADE
 ) ENGINE=InnoDB";
 
-		$query[] = "CREATE TABLE `MuninGraph` (
-  `object_id` int(10) unsigned NOT NULL,
-  `server_id` int(10) unsigned NOT NULL,
-  `graph` char(255) NOT NULL,
-  `caption`  char(255) DEFAULT NULL,
-  PRIMARY KEY (`object_id`,`server_id`,`graph`),
-  KEY `server_id` (`server_id`),
-  KEY `graph` (`graph`),
-  CONSTRAINT `MuninGraph-FK-server_id` FOREIGN KEY (`server_id`) REFERENCES `MuninServer` (`id`),
-  CONSTRAINT `MuninGraph-FK-object_id` FOREIGN KEY (`object_id`) REFERENCES `Object` (`id`) ON DELETE CASCADE
-) ENGINE=InnoDB";
-
-		$query[] = "CREATE TABLE `MuninServer` (
-  `id` int(10) unsigned NOT NULL auto_increment,
-  `base_url` char(255) DEFAULT NULL,
-  PRIMARY KEY (`id`)
-) ENGINE=InnoDB";
-
 		$query[] = "CREATE TABLE `ObjectLog` (
   `id` int(10) unsigned NOT NULL AUTO_INCREMENT,
   `object_id` int(10) unsigned NOT NULL,
@@ -929,6 +887,15 @@ function get_pseudo_file ($name)
   UNIQUE KEY `pctype_per_origin` (`pctype`,`origin`)
 ) ENGINE=InnoDB";
 
+		$query[] = "CREATE TABLE `Plugin` (
+  `name` char(255) NOT NULL,
+  `longname` char(255) NOT NULL,
+  `version` char(64) NOT NULL,
+  `home_url` char(255) NOT NULL,
+  `state` enum('disabled','enabled') NOT NULL default 'disabled',
+  PRIMARY KEY (`name`)
+) ENGINE=InnoDB";
+
 		$query[] = "CREATE TABLE `Port` (
   `id` int(10) unsigned NOT NULL auto_increment,
   `object_id` int(10) unsigned NOT NULL default '0',
@@ -1032,6 +999,7 @@ function get_pseudo_file ($name)
 ) ENGINE=InnoDB";
 
 		$query[] = "CREATE TABLE `ObjectHistory` (
+  `event_id` int(10) unsigned NOT NULL AUTO_INCREMENT,
   `id` int(10) unsigned default NULL,
   `name` char(255) default NULL,
   `label` char(255) default NULL,
@@ -1041,6 +1009,7 @@ function get_pseudo_file ($name)
   `comment` text,
   `ctime` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
   `user_name` char(64) default NULL,
+  PRIMARY KEY (`event_id`),
   KEY `id` (`id`),
   CONSTRAINT `ObjectHistory-FK-object_id` FOREIGN KEY (`id`) REFERENCES `Object` (`id`) ON DELETE CASCADE
 ) ENGINE=InnoDB";
@@ -1089,6 +1058,8 @@ function get_pseudo_file ($name)
   `parent_id` int(10) unsigned default NULL,
   `is_assignable` enum('yes','no') NOT NULL DEFAULT 'yes',
   `tag` char(255) default NULL,
+  `color` mediumint(8) unsigned DEFAULT NULL,
+  `description` char(255) COLLATE utf8_unicode_ci DEFAULT NULL,
   PRIMARY KEY  (`id`),
   UNIQUE KEY `tag` (`tag`),
   KEY `TagTree-K-parent_id` (`parent_id`),
@@ -1344,7 +1315,7 @@ EntityLinkTrigger:BEGIN
   END IF;
 END;
 ";
-		$link_trigger_body = <<<ENDOFTRIGGER
+		$link_trigger_body = <<<'ENDOFTRIGGER'
 LinkTrigger:BEGIN
   DECLARE tmp, porta_type, portb_type, count INTEGER;
 
@@ -1441,7 +1412,7 @@ WHERE O.objtype_id = 1562";
 (29,'uint','Sort order'),
 (30,'dict','Mgmt type'),
 -- ^^^^^ Any new 'default' attributes must go above this line! ^^^^^
--- Primary key value 9999 makes sure, that AUTO_INCREMENT on server restart
+-- Primary key value 9999 makes sure that AUTO_INCREMENT on server restart
 -- doesn't drop below 10000 (other code relies on this, site-specific
 -- attributes are assigned IDs starting from 10000).
 (9999,'string','base MAC address')";
@@ -1473,6 +1444,7 @@ WHERE O.objtype_id = 1562";
 (36,'no','serial console server models'),
 (37,'no','wireless OS type'),
 (38,'no','management interface type'),
+(39,'no','UPS models'),
 -- Default chapters must have ID less than 10000, add them above this line.
 (9999,'no','multiplexer models')";
 
@@ -1621,7 +1593,8 @@ WHERE O.objtype_id = 1562";
 (1644,3,NULL,'no'),
 (1787,3,NULL,'no'),
 (1787,14,NULL,'no'),
-(1787,30,38,'yes')";
+(1787,30,38,'yes'),
+(12,2,39,'no')";
 
 		$query[] = "INSERT INTO PatchCableConnector (id, origin, connector) VALUES
 (1,'default','FC/PC'),(2,'default','FC/APC'),
@@ -1698,7 +1671,8 @@ WHERE O.objtype_id = 1562";
 (12,'CFP2'),
 (13,'CPAK'),
 (14,'CXP'),
-(15,'QSFP28')";
+(15,'QSFP28'),
+(16,'SFP28')";
 
 		$query[] = "INSERT INTO `PortOuterInterface` VALUES
 (16,'AC-in'),
@@ -1894,9 +1868,16 @@ WHERE O.objtype_id = 1562";
 (1589,'empty CFP2'),
 (1590,'empty CPAK'),
 (1591,'empty CXP'),
+(1592,'empty SFP28'),
 (1603,'1000Base-T (HP c-Class)'),
 (1604,'100Base-TX (HP c-Class)'),
 (1642,'10GBase-T'),
+(1651,'25GBase-KR'),
+(1652,'25GBase-T'),
+(1653,'25GBase-CR'),
+(1654,'25GBase-SR'),
+(1655,'25GBase-LR'),
+(1656,'25Gbase-ER'),
 (1660,'40GBase-FR'),
 (1661,'40GBase-KR4'),
 (1662,'40GBase-ER4'),
@@ -2007,6 +1988,10 @@ WHERE O.objtype_id = 1562";
 (14,1591),(14,1677),(14,1678),
 -- QSFP28: empty QSFP, 40GBase-FR, 40GBase-ER4, 40GBase-SR4, 40GBase-LR4, 100GBase-LR4, 100GBase-ER4, 100GBase-SR4, 100GBase-KR4, 100GBase-KP4
 (15,1588),(15,1660),(15,1662),(15,1663),(15,1664),(15,1670),(15,1671),(15,1672),(15,1673),(15,1674),
+-- SFP28: empty SFP28, 25Gbase-KR, 25GBase-CR, 25Gbase-SR, 25GBase-LR, 25GBase-ER
+(16,1592),(16,1651),(16,1653),(16,1654),(16,1655),(16,1656),
+-- SFP28: 10GBase-SR, 10GBase-ER, 10GBase-LR, 10GBase-LRM, 10GBase-ZR, 10GBase-LX4, 10GBase-CX4
+(16,30),(16,35),(16,36),(16,37),(16,38),(16,39),(16,40),
 -- hardwired: AC-in, 100Base-TX, 1000Base-T, RS-232 (RJ-45), virtual bridge, KVM (host), KVM (console), RS-232 (DB-9), RS-232 (DB-25), AC-out, DC, virtual port
 (1,16),(1,19),(1,24),(1,29),(1,31),(1,33),(1,446),(1,681),(1,682),(1,1322),(1,1399),(1,1469)";
 
@@ -2199,7 +2184,15 @@ WHERE O.objtype_id = 1562";
 (1589,1590),
 (1590,1590),
 (1591,1591),
+(1592,1592),
+(1592,1084),
 (1603,1603),
+(1651,1651),
+(1652,1652),
+(1653,1653),
+(1654,1654),
+(1655,1655),
+(1656,1656),
 (1660,1660),
 (1661,1661),
 (1662,1662),
@@ -2271,11 +2264,11 @@ WHERE O.objtype_id = 1562";
 ('TAGS_QUICKLIST_THRESHOLD','50','uint','yes','no','yes','Tags quick list threshold'),
 ('ENABLE_MULTIPORT_FORM','no','string','no','no','yes','Enable \"Add/update multiple ports\" form'),
 ('DEFAULT_PORT_IIF_ID','1','uint','no','no','no','Default port inner interface ID'),
-('DEFAULT_PORT_OIF_IDS','1=24; 3=1078; 4=1077; 5=1079; 6=1080; 8=1082; 9=1084; 10=1588; 11=1668; 12=1589; 13=1590; 14=1591; 15=1588','string','no','no','no','Default port outer interface IDs'),
+('DEFAULT_PORT_OIF_IDS','1=24; 3=1078; 4=1077; 5=1079; 6=1080; 8=1082; 9=1084; 10=1588; 11=1668; 12=1589; 13=1590; 14=1591; 15=1588; 16=1592','string','no','no','no','Default port outer interface IDs'),
 ('IPV4_TREE_RTR_AS_CELL','no','string','no','no','yes','Show full router info for each network in IPv4 tree view'),
 ('PROXIMITY_RANGE','0','uint','yes','no','yes','Proximity range (0 is current rack only)'),
 ('VLANSWITCH_LISTSRC', '', 'string', 'yes', 'no', 'yes', 'List of VLAN running switches'),
-('VLANIPV4NET_LISTSRC', '', 'string', 'yes', 'no', 'yes', 'List of VLAN-based IPv4 networks'),
+('VLANNET_LISTSRC', '', 'string', 'yes', 'no', 'yes', 'List of VLAN-related IPv4/IPv6 networks'),
 ('IPV4_TREE_SHOW_VLAN','yes','string','no','no','yes','Show VLAN for each network in IPv4 tree'),
 ('DEFAULT_VDOM_ID','','uint','yes','no','yes','Default VLAN domain ID'),
 ('DEFAULT_VST_ID','','uint','yes','no','yes','Default VLAN switch template ID'),
@@ -2296,18 +2289,17 @@ WHERE O.objtype_id = 1562";
 ('MGMT_PROTOS','ssh: {\$typeid_4}; telnet: {\$typeid_8}','string','yes','no','yes','Mapping of management protocol to devices'),
 ('SYNC_8021Q_LISTSRC','','string','yes','no','no','List of VLAN switches sync is enabled on'),
 ('QUICK_LINK_PAGES','depot,ipv4space,rackspace','string','yes','no','yes','List of pages to display in quick links'),
-('CACTI_LISTSRC','false','string','yes','no','no','List of object with Cacti graphs'),
-('CACTI_RRA_ID','1','uint','no','no','yes','RRA ID for Cacti graphs displayed in RackTables'),
-('MUNIN_LISTSRC','false','string','yes','no','no','List of object with Munin graphs'),
-('VIRTUAL_OBJ_LISTSRC','1504,1505,1506,1507','string','no','no','no','List source: virtual objects'),
+('VIRTUAL_OBJ_CSV','1504,1505,1506,1507','string','no','no','no','List source: virtual objects'),
 ('DATETIME_ZONE','UTC','string','yes','no','yes','Timezone to use for displaying/calculating dates'),
-('DATETIME_FORMAT','%Y-%m-%d','string','no','no','yes','PHP strftime() format to use for date output'),
+('DATETIME_FORMAT','%Y-%m-%d','string','no','no','yes','PHP strftime() format for date+time'),
+('DATEONLY_FORMAT','%Y-%m-%d','string','no','no','yes','PHP strftime() format for dates'),
 ('SEARCH_DOMAINS','','string','yes','no','yes','DNS domain list (comma-separated) to search in FQDN attributes'),
 ('8021Q_EXTSYNC_LISTSRC','false','string','yes','no','no','List source: objects with extended 802.1Q sync'),
 ('8021Q_MULTILINK_LISTSRC','false','string','yes','no','no','List source: IPv4/IPv6 networks allowing multiple VLANs from same domain'),
 ('REVERSED_RACKS_LISTSRC', 'false', 'string', 'yes', 'no', 'no', 'List of racks with reversed (top to bottom) units order'),
 ('NEAREST_RACKS_CHECKBOX', 'yes', 'string', 'yes', 'no', 'yes', 'Enable nearest racks in port list filter by default'),
 ('SHOW_OBJECTTYPE', 'yes', 'string', 'no', 'no', 'yes', 'Show object type column on depot page'),
+('OBJECTLOG_PREVIEW_ENTRIES','5','uint','no','no','yes','Object log preview maximum entries (0 disables the preview)'),
 ('DB_VERSION','${db_version}','string','no','yes','no','Database version.')";
 
 		$query[] = "INSERT INTO `Script` VALUES ('RackCode','allow {\$userid_1}')";
@@ -2327,5 +2319,3 @@ WHERE O.objtype_id = 1562";
 	return $query;
 	}
 }
-
-?>
