@@ -6,7 +6,7 @@
 
 $relnotes = array
 (
-	'0.20.0' => <<<ENDOFTEXT
+	'0.20.0' => <<<'ENDOFTEXT'
 WARNING: This release have too many internal changes, some of them were waiting more than a year
 to be released. So this release is considered "BETA" and is recommended only to curiuos users,
 who agree to sacrifice the stability to the progress.
@@ -19,17 +19,17 @@ New plugins engine instead of local.php file. To make your own code stored in lo
 you must move the local.php file into the plugins/ directory. The name of this file does not
 matter any more. You also can store multiple files in that dir, separate your plugins by features,
 share them and try the plugins from other people just placing them into plugins/ dir, no more merging.
-\$path_to_local_php variable has no special meaning any more.
-\$racktables_confdir variable is now used only to search for secret.php file.
-\$racktables_plugins_dir is a new overridable special variable pointing to plugins/ directory.
+$path_to_local_php variable has no special meaning any more.
+$racktables_confdir variable is now used only to search for secret.php file.
+$racktables_plugins_dir is a new overridable special variable pointing to plugins/ directory.
 
 Beginning with this version it is possible to delete IP prefixes, VLANs, Virtual services
 and RS pools from within theirs properties tab. So please inspect your permissions rules
 to assure there are no undesired allows for deletion of these objects. To ensure this, you could
 try this code in the beginning of permissions script:
 
-	allow {userid_1} and {\$op_del}
-	deny {\$op_del} and ({\$tab_edit} or {\$tab_properties})
+	allow {userid_1} and {$op_del}
+	deny {$op_del} and ({$tab_edit} or {$tab_properties})
 
 Hardware gateways engine was rewritten in this version of RackTables. This means that
 the file gateways/deviceconfig/switch.secrets.php is not used any more. To get information
@@ -41,9 +41,9 @@ gateway support contributed by Ilya Evseev) are not working any more and waiting
 forward-ported to new gateways API. Sorry for that.
 
 Two new config variables appeared in this version:
-  - SEARCH_DOMAINS. Comma-separated list of DNS domains which are considered "base" for your
+  - SEARCH_DOMAINS. Comma-separated list of DNS domains that are considered "base" for your
     network. If RackTables search engine finds multiple objects based on your search input, but
-    there is only one which FQDN consists of your input and one of these search domains, you will
+    there is only one that has FQDN consisting of your input and one of these search domains, you will
     be redirected to this object and other results will be discarded. Such behavior was unconditional
     since 0.19.3, which caused many objections from users. So welcome this config var.
   - QUICK_LINK_PAGES. Comma-separated list of RackTables pages to display links to them on top.
@@ -55,7 +55,7 @@ This could be inconvenient, but it is the most effective way to encourage users
 If this behavior is not what you want, simply revert these variables' values:
   - SHOW_LAST_TAB               no => yes
   - IPV4_TREE_SHOW_USAGE        yes =>no (networks' usage is still available by click)
-  - IPV4LB_LISTSRC              {\$typeid_4} => false
+  - IPV4LB_LISTSRC              {$typeid_4} => false
   - FILTER_DEFAULT_ANDOR        or => and (this implicitly enables the feature of dynamic tree shrinking)
   - FILTER_SUGGEST_EXTRA        no => yes (yes, we have extra logical filters!)
   - IPV4_TREE_RTR_AS_CELL       yes => no (display routers as simple text, not cell)
@@ -69,18 +69,18 @@ IP tree at all.
 ENDOFTEXT
 ,
 
-	'0.20.1' => <<<ENDOFTEXT
-The 0.20.0 release includes bug which breaks IP networks' capacity displaying on 32-bit architecture machines. To fix this, this release makes use of PHP's BC Math module. It is a new reqiurement. Most PHP distributions have this module already enabled, but if yours does not - you need yo recompile PHP.
+	'0.20.1' => <<<'ENDOFTEXT'
+The 0.20.0 release includes a bug that breaks IP networks' capacity displaying on 32-bit architecture machines. To fix this, this release makes use of PHP's BC Math module. It is a new reqiurement. Most PHP distributions have this module already enabled, but if yours does not - you need yo recompile PHP.
 
 Security context of 'ipaddress' page now includes tags from the network containing an IP address. This means that you should audit your permission rules to check there is no unintended allows of changing IPs based on network's tagset. Example:
 	allow {client network} and {New York}
 This rule now not only allows any operation on NY client networks, but also any operation with IP addresses included in those networks. To fix this, you should change the rule this way:
-	allow {client network} and {New York} and not {\$page_ipaddress}
+	allow {client network} and {New York} and not {$page_ipaddress}
 
 ENDOFTEXT
 ,
 
-	'0.20.4' => <<<ENDOFTEXT
+	'0.20.4' => <<<'ENDOFTEXT'
 Please note that some dictionary items of Cisco Catalyst 2960 series switches
 were renamed to meet official Cisco classification:
 	2960-48TT   => 2960-48TT-L
@@ -104,7 +104,7 @@ REVERSED_RACKS_LISTSRC and NEAREST_RACKS_CHECKBOX.
 ENDOFTEXT
 ,
 
-	'0.20.5' => <<<ENDOFTEXT
+	'0.20.5' => <<<'ENDOFTEXT'
 This release introduces the VS groups feature. VS groups is a new way to store
 and display virtual services configuration. There is a new "ipvs" (VS group)
 realm. All previously existing VS configuration remains functional and user
@@ -121,17 +121,17 @@ new format.
 ENDOFTEXT
 ,
 
-	'0.20.6' => <<<ENDOFTEXT
+	'0.20.6' => <<<'ENDOFTEXT'
 New MGMT_PROTOS configuration option replaces the TELNET_OBJS_LISTSRC,
 SSH_OBJS_LISTSRC and RDP_OBJS_LISTSRC options (converting existing settings as
 necessary). MGMT_PROTOS allows to specify any management protocol for a
 particular device list using a RackCode filter. The default value
-("ssh: {\$typeid_4}, telnet: {\$typeid_8}") produces "ssh://server.fqdn" for
+("ssh: {$typeid_4}, telnet: {$typeid_8}") produces "ssh://server.fqdn" for
 servers and "telnet://switch.fqdn" for network switches.
 ENDOFTEXT
 ,
 
-	'0.20.7' => <<<ENDOFTEXT
+	'0.20.7' => <<<'ENDOFTEXT'
 From now on the minimum (oldest) release of PHP that can run RackTables is
 5.2.10. In particular, to continue running RackTables on CentOS 5 it is
 necessary to replace its php* RPM packages with respective php53* packages
@@ -141,14 +141,14 @@ Database triggers are used for some data consistency measures.  The database
 user account must have the 'TRIGGER' privilege, which was introduced in
 MySQL 5.1.7.
 
-The IPV4OBJ_LISTSRC configuration option is reset to an expression which enables
+The IPV4OBJ_LISTSRC configuration option is reset to an expression that enables
 the IP addressing feature for all object types except those listed.
 
 Tags could now be assigned on the Edit/Properties tab using a text input with
 auto-completion. Type a star '*' to view full tag tree in auto-complete menu.
 It is worth to add the following line to the permissions script if the
 old-fashioned 'Tags' tab is not needed any more:
-  deny {\$tab_tags} # this hides 'Tags' tab
+  deny {$tab_tags} # this hides 'Tags' tab
 
 This release converts collation of all DB fields to the utf8_unicode_ci. This
 procedure may take some time, and could fail if there are rows that differ only
@@ -158,19 +158,41 @@ installation. If desired so, you could eliminate the case-duplicating rows
 and re-apply the failed query.
 ENDOFTEXT
 ,
-	'0.20.11' => <<<ENDOFTEXT
+	'0.20.11' => <<<'ENDOFTEXT'
 New IPV4_TREE_SHOW_UNALLOCATED configuration option introduced to disable
 dsplaying unallocated networks in IPv4 space tree. Setting it also disables
 the "knight" feature.
 ENDOFTEXT
+,
+	'0.21.0' => <<<'ENDOFTEXT'
+From now on the minimum (oldest) release of PHP that can run RackTables is
+5.5.0.
+
+This release introduces a new plugin architecture.  If you experience issues
+after the upgrade, try disabling plugins.
+Refer to <a href="http://wiki.racktables.org/index.php/Plugins">the wiki</a> for more information.
+ENDOFTEXT
+,
+	'0.21.2' => <<<'ENDOFTEXT'
+"Shared router" allocation type introduced, useful for documenting VRRP-protected addresses.
+
+This version drops support for the $localreports global variable, which is
+trivial to replace in a local plugin if necessary.
+
+The "addJS()" function is now deprecated in favour of "addJSText()", "addJSInternal()",
+and "addJSExternal()" functions.  The "addJS()" function will likely be removed in 0.22.0.
+
+The "addCSS()" function is now deprecated in favour of "addCSSText()", "addCSSInternal()",
+and "addCSSExternal()" functions.  The "addCSS()" function will likely be removed in 0.22.0.
+
+For more information on the "addJS()" and "addCSS()" changes see the README.md
+ENDOFTEXT
 ,
 );
 
-// At the moment we assume, that for any two releases we can
-// sequentally execute all batches, that separate them, and
-// nothing will break. If this changes one day, the function
-// below will have to generate smarter upgrade paths, while
-// the upper layer will remain the same.
+// This function currently implements a convention that to upgrade any release R1
+// to a release R2 (such that R2 > R1) it takes to apply the sequence of batches
+// (R1, R2] to database R1.
 // Returning an empty array means that no upgrade is necessary.
 // Returning NULL indicates an error.
 function getDBUpgradePath ($v1, $v2)
@@ -192,6 +214,12 @@ function getDBUpgradePath ($v1, $v2)
 		'0.20.12',
 		'0.20.13',
 		'0.20.14',
+		'0.21.0',
+		'0.21.1',
+		'0.21.2',
+		'0.21.3',
+		'0.21.4',
+		'0.21.5',
 	);
 	if (! in_array ($v1, $versionhistory) || ! in_array ($v2, $versionhistory))
 		return NULL;
@@ -390,7 +418,7 @@ CREATE VIEW `RackObject` AS SELECT id, name, label, objtype_id, asset_no, has_pr
 
 			$query[] = "INSERT INTO `Config` (varname, varvalue, vartype, emptyok, is_hidden, is_userdefined, description) VALUES ('SEARCH_DOMAINS','','string','yes','no','yes','DNS domain list (comma-separated) to search in FQDN attributes')";
 
-			// update some config variables which changed their defaults in this verison
+			// update some config variables that changed their defaults in this verison
 			replaceConfigVarValue ('SHOW_LAST_TAB', 'yes');
 			replaceConfigVarValue ('IPV4_TREE_SHOW_USAGE','no');
 			replaceConfigVarValue ('IPV4LB_LISTSRC', 'false', '{$typeid_4}');
@@ -726,7 +754,7 @@ EntityLinkTrigger:BEGIN
   END IF;
 END;
 ";
-			$link_trigger_body = <<<ENDOFTRIGGER
+			$link_trigger_body = <<<'ENDOFTRIGGER'
 LinkTrigger:BEGIN
   DECLARE tmp, porta_type, portb_type, count INTEGER;
 
@@ -1178,7 +1206,7 @@ VALUES ('SHOW_OBJECTTYPE',  'no',  'string',  'no',  'no',  'yes',  'Show object
 
 			$query[] = "INSERT INTO ObjectParentCompat (parent_objtype_id, child_objtype_id) VALUES (1787,4)";
 
-			$port_trigger_body = <<<ENDOFTRIGGER
+			$port_trigger_body = <<<'ENDOFTRIGGER'
 PortTrigger:BEGIN
   IF (NEW.`l2address` IS NOT NULL AND (SELECT COUNT(*) FROM `Port` WHERE `l2address` = NEW.`l2address` AND `object_id` != NEW.`object_id`) > 0) THEN
     CALL `Port-l2address-already-exists-on-another-object`;
@@ -1215,7 +1243,6 @@ ENDOFTRIGGER;
 				(11,1088), (12,1088), (11,1089), (12,1089),
 				(11,1090), (12,1090), (11,1091), (12,1091)";
 
-
 			$query[] = "UPDATE Config SET varvalue = '0.20.12' WHERE varname = 'DB_VERSION'";
 			break;
 		case '0.20.13':
@@ -1228,6 +1255,116 @@ ENDOFTRIGGER;
 			$query[] = "UPDATE ObjectHistory SET ctime = ctime, has_problems = 'no' WHERE objtype_id = 1561 AND has_problems = ''";
 			$query[] = "UPDATE Config SET varvalue = '0.20.14' WHERE varname = 'DB_VERSION'";
 			break;
+		case '0.21.0':
+			$query[] = "UPDATE Port SET label = NULL WHERE label = ''";
+			$query[] = "DELETE FROM RackThumbnail";
+			$query[] = "ALTER TABLE TagTree ADD COLUMN color mediumint unsigned DEFAULT NULL AFTER tag";
+
+			$query[] = "
+CREATE TABLE `Plugin` (
+  `name` char(255) NOT NULL,
+  `longname` char(255) NOT NULL,
+  `version` char(64) NOT NULL,
+  `home_url` char(255) NOT NULL,
+  `state` enum('disabled','enabled') NOT NULL default 'disabled',
+  PRIMARY KEY (`name`)
+) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci";
+
+			// if the Cacti or Munin plugins are being used, mark them as enabled, otherwise uninstall them
+			// from now on, upgrades will be handled by the plugins themselves
+			$result = $dbxlink->query ('SELECT COUNT(*) FROM CactiServer');
+			if ($result->fetchColumn() > 0)
+				$query[] = "INSERT INTO Plugin VALUES ('cacti','Cacti','1.0','http://www.racktables.org/','enabled')";
+			else
+			{
+				$query[] = "DELETE FROM Config WHERE varname IN ('CACTI_LISTSRC','CACTI_RRA_ID')";
+				$query[] = "UPDATE Config SET varname = 'VLANNET_LISTSRC' WHERE varname = 'VLANIPV4NET_LISTSRC'";
+				$query[] = "DROP TABLE `CactiGraph`";
+				$query[] = "DROP TABLE `CactiServer`";
+			}
+			unset ($result);
+			$result = $dbxlink->query ('SELECT COUNT(*) FROM MuninServer');
+			if ($result->fetchColumn() > 0)
+				$query[] = "INSERT INTO Plugin VALUES ('munin','Munin','1.0','http://www.racktables.org/','enabled')";
+			else
+			{
+				$query[] = "DELETE FROM Config WHERE varname = 'MUNIN_LISTSRC'";
+				$query[] = "DROP TABLE `MuninGraph`";
+				$query[] = "DROP TABLE `MuninServer`";
+			}
+			unset ($result);
+
+			$query[] = "UPDATE Config SET description = 'PHP strftime() format for date+time' WHERE varname = 'DATETIME_FORMAT'";
+			$query[] = "
+INSERT INTO `Config` (varname, varvalue, vartype, emptyok, is_hidden, is_userdefined, description) VALUES
+('DATEONLY_FORMAT','%Y-%m-%d','string','no','no','yes','PHP strftime() format for dates')";
+
+			$query[] = "UPDATE Config SET varvalue = '0.21.0' WHERE varname = 'DB_VERSION'";
+			break;
+		case '0.21.1':
+			// The query below was conditional in the 0.20.0 section, which was wrong.
+			$query[] = "UPDATE Config SET varname = 'VLANNET_LISTSRC' WHERE varname = 'VLANIPV4NET_LISTSRC'";
+			$query[] = "UPDATE Config SET description = 'List of VLAN-related IPv4/IPv6 networks' WHERE varname = 'VLANNET_LISTSRC'";
+			$query[] = "ALTER TABLE ObjectHistory ADD COLUMN event_id int(10) unsigned NOT NULL AUTO_INCREMENT FIRST, ADD PRIMARY KEY (event_id)";
+			$query[] = "DELETE FROM Atom WHERE molecule_id IS NULL OR rack_id IS NULL OR unit_no IS NULL OR atom IS NULL";
+			// Modify the columns explicitly before the primary key addition does it implicitly.
+			$query[] = "ALTER TABLE Atom MODIFY COLUMN molecule_id int(10) unsigned NOT NULL";
+			$query[] = "ALTER TABLE Atom MODIFY COLUMN rack_id int(10) unsigned NOT NULL";
+			$query[] = "ALTER TABLE Atom MODIFY COLUMN unit_no int(10) unsigned NOT NULL";
+			$query[] = "ALTER TABLE Atom MODIFY COLUMN atom enum('front','interior','rear') NOT NULL";
+			$query[] = "ALTER TABLE Atom ADD PRIMARY KEY (molecule_id, rack_id, unit_no, atom)";
+			$query[] = "ALTER TABLE Atom DROP KEY `Atom-FK-molecule_id`";
+			$query[] = "UPDATE Config SET varvalue = '0.21.1' WHERE varname = 'DB_VERSION'";
+			break;
+		case '0.21.2':
+			$query[] = "ALTER TABLE MountOperation ADD UNIQUE KEY `old_molecule_id` (old_molecule_id)";
+			$query[] = "ALTER TABLE MountOperation ADD UNIQUE KEY `new_molecule_id` (new_molecule_id)";
+			$query[] = "ALTER TABLE MountOperation DROP KEY `MountOperation-FK-old_molecule_id`";
+			$query[] = "ALTER TABLE MountOperation DROP KEY `MountOperation-FK-new_molecule_id`";
+			$query[] = "ALTER TABLE IPv4Allocation MODIFY type ENUM('regular','shared','virtual','router','point2point','sharedrouter') NOT NULL DEFAULT 'regular'";
+			$query[] = "ALTER TABLE IPv6Allocation MODIFY type ENUM('regular','shared','virtual','router','point2point','sharedrouter') NOT NULL DEFAULT 'regular'";
+			$query[] = "INSERT INTO Chapter (`id`, `sticky`, `name`) VALUES (39,'no','UPS models')";
+			$query[] = "INSERT INTO AttributeMap (`objtype_id`,`attr_id`,`chapter_id`) VALUES (12,2,39)"; // UPS (UPS models) -> HW type
+			$query[] = "ALTER TABLE TagTree ADD COLUMN description char(255) DEFAULT NULL AFTER color";
+
+			// new iif_type SFP28
+			$query[] = "INSERT INTO PortInnerInterface VALUES (16, 'SFP28')";
+			$query[] = "INSERT INTO PortOuterInterface VALUES
+				(1592, 'empty SFP28'),
+				(1651, '25GBase-KR'),
+				(1652, '25GBase-T'),
+				(1653, '25GBase-CR'),
+				(1654, '25GBase-SR'),
+				(1655, '25GBase-LR'),
+				(1656, '25Gbase-ER')";
+			$query[] = "INSERT INTO PortInterfaceCompat VALUES
+				(16,1592),(16,1651),(16,1653),(16,1654),(16,1655),(16,1656),
+				(16,30),(16,35),(16,36),(16,37),(16,38),(16,39),(16,40)";
+			$query[] = "INSERT INTO PortCompat VALUES
+				(1592, 1592),
+				(1592, 1084),
+				(1084, 1592),
+				(1651, 1651),
+				(1652, 1652),
+				(1653, 1653),
+				(1654, 1654),
+				(1655, 1655),
+				(1656, 1656)";
+			$query[] = "UPDATE Config SET varvalue = CONCAT(varvalue, '; 16=1592')
+				WHERE varname = 'DEFAULT_PORT_OIF_IDS' AND 0 = INSTR(varvalue, '16=')";
+			$query[] = "UPDATE Config SET varname = 'VIRTUAL_OBJ_CSV' WHERE varname = 'VIRTUAL_OBJ_LISTSRC'";
+			$query[] = "UPDATE Config SET varvalue = '0.21.2' WHERE varname = 'DB_VERSION'";
+			break;
+		case '0.21.3':
+			$query[] = "INSERT INTO Config VALUES ('OBJECTLOG_PREVIEW_ENTRIES','5','uint','no','no','yes','Object log preview maximum entries (0 disables the preview)')";
+			$query[] = "UPDATE Config SET varvalue = '0.21.3' WHERE varname = 'DB_VERSION'";
+			break;
+		case '0.21.4':
+			$query[] = "UPDATE Config SET varvalue = '0.21.4' WHERE varname = 'DB_VERSION'";
+			break;
+		case '0.21.5':
+			$query[] = "UPDATE Config SET varvalue = '0.21.5' WHERE varname = 'DB_VERSION'";
+			break;
 		case 'dictionary':
 			$query = reloadDictionary();
 			break;
@@ -1360,17 +1497,18 @@ function renderUpgraderHTML()
 	{
 		header ('WWW-Authenticate: Basic realm="RackTables upgrade"');
 		header ('HTTP/1.0 401 Unauthorized');
-?>
+		echo <<<'ENDOFTEXT'
 <h1>Trouble logging in?</h1>
 You are trying to authenticate for the RackTables upgrade screen. This means that
 you must authenticate with the username and password of the main RackTables
 administrator. There is only one such account in each installation, its default
 username is "admin". RackTables wiki provides more information on this topic.
-<?php
+ENDOFTEXT;
 		die;
 	}
 
-?><!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
+		echo <<<'ENDOFTEXT'
+<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
 <html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
 <head><title>RackTables upgrade script</title>
 <style type="text/css">
@@ -1393,9 +1531,9 @@ username is "admin". RackTables wiki provides more information on this topic.
 </head>
 <body>
 <h1>Platform check status</h1>
-<?php
+ENDOFTEXT;
 
-if (!platform_is_ok())
+if (! platform_is_ok (TRUE))
 {
 	echo '<h1>Please resolve the failed (red) item(s) above.</h1>';
 	die ('</body></html>');
@@ -1440,7 +1578,7 @@ else
 			echo 'It is <strong>highly recommended</strong> to make a database backup before ';
 			echo 'proceeding any further. <tt>mysqldump</tt> and <tt>PHPMyAdmin</tt> are convenient ';
 			echo 'tools for doing this.</p>';
-			echo '<p><input type=checkbox name=reallyreally id=reallyreally><label for=reallyreally>';
+			echo '<p><label><input type=checkbox name=reallyreally>';
 			echo 'I am ready to bear all risks of this upgrade. I am ready to roll it back in case of ';
 			echo 'a failure.</label> <input type=submit value="Yes, I am."></p></td></tr></form>';
 		}
@@ -1463,7 +1601,7 @@ function convertSLBTablesToBinIPs()
 
 	$dbxlink->query ("DROP TABLE IF EXISTS `IPv4VS_new`, `IPv4RS_new`, `IPv4VS_old`, `IPv4RS_old`");
 
-	$dbxlink->query (<<<END
+	$dbxlink->query (<<<'END'
 CREATE TABLE `IPv4VS_new` (
   `id` int(10) unsigned NOT NULL auto_increment,
   `vip` varbinary(16) NOT NULL,
@@ -1486,7 +1624,7 @@ END
 		usePreparedInsertBlade ('IPv4VS_new', $row);
 	}
 
-	$dbxlink->query (<<<END
+	$dbxlink->query (<<<'END'
 CREATE TABLE `IPv4RS_new` (
   `id` int(10) unsigned NOT NULL auto_increment,
   `inservice` enum('yes','no') NOT NULL default 'no',
@@ -1511,7 +1649,7 @@ END
 		usePreparedInsertBlade ('IPv4RS_new', $row);
 	}
 
-	$dbxlink->query (<<<END
+	$dbxlink->query (<<<'END'
 RENAME TABLE
 	`IPv4VS` TO `IPv4VS_old`,
 	`IPv4VS_new` TO `IPv4VS`,
@@ -1547,5 +1685,3 @@ function convertMgmtConfigVars()
 	}
 	return implode (',', $ret);
 }
-
-?>
